-- ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-- Written by Kimberly L. Tripp - all rights reserved.
-- 
-- For more scripts and sample code, check out 
-- 	http://www.SQLSkills.com
-- For "More than Just Training," see the Industry Experts at 
-- 	http://www.SolidQualityLearning.com
--
-- Disclaimer - Thoroughly test this script, EXECute at your own risk.
-- ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

USE Credit
go

SET STATISTICS IO ON
-- Turn Graphical Showplan ON (Ctrl+K)

CREATE INDEX MemberLastName ON dbo.member(LastName)
CREATE INDEX MemberFirstName ON dbo.member(FirstName)
go

UPDATE Member
SET LastName = 'Tripp'
WHERE Member_No = 6798
go

UPDATE Member
SET FirstName = 'Kimberly'
WHERE Member_No = 2896
go

UPDATE Member
SET FirstName = 'Kimberly',
	LastName = 'Tripp'
WHERE Member_No = 842
go
IF OBJECTPROPERTY(object_id('dbo.GetMemberInfoParam'), 'IsProcedure') = 1
	DROP PROC dbo.GetMemberInfoParam
IF OBJECTPROPERTY(object_id('dbo.GetMemberInfoParam2'), 'IsProcedure') = 1
	DROP PROC dbo.GetMemberInfoParam2
IF OBJECTPROPERTY(object_id('dbo.GetMemberInfoParamDSE'), 'IsProcedure') = 1
	DROP PROC dbo.GetMemberInfoParamDSE
go
CREATE PROC dbo.GetMemberInfoParam
	@Lastname	varchar(30) = NULL,
	@Firstname	varchar(30) = NULL,
	@member_no	int = NULL
AS
SELECT * FROM dbo.member
WHERE (lastname LIKE @lastname OR @lastname IS NULL)
	AND (member_no = @member_no OR @member_no IS NULL)
	AND (firstname LIKE @firstname OR @firstname IS NULL)
go
EXEC dbo.GetMemberInfoParam @Lastname = 'Tripp' with recompile
go
EXEC dbo.GetMemberInfoParam @Firstname = 'Kimberly' with recompile
go
EXEC dbo.GetMemberInfoParam @Member_no = 9912 --with recompile
go
-- Another look...
-- By re-arranging the parameters and putting the variable test first
-- SQL Server does a *better* (but more generalized) job analyzing 
-- the expression (because parens have OR and all the ORs are 
-- AND - meaning that ALL conditions must be true (AND) but only 
-- ONE from each of the expressions must be true (OR). 
CREATE PROC dbo.GetMemberInfoParam2
	@Lastname	varchar(30) = NULL,
	@Firstname	varchar(30) = NULL,
	@member_no	int = NULL
AS
SELECT * FROM dbo.member
WHERE (@lastname IS NULL OR lastname LIKE @lastname)
	AND (@member_no  IS NULL OR member_no = @member_no)
	AND (@firstname  IS NULL OR firstname LIKE @firstname)
go
EXEC dbo.GetMemberInfoParam2 @Lastname = 'Tripp' with recompile
go
EXEC dbo.GetMemberInfoParam2 @Firstname = 'Kimberly' with recompile
go
EXEC dbo.GetMemberInfoParam2 @Member_no = 9912 --with recompile
go
-- BUT none of the first params help the optmizer for the table 
-- SO SQL Server chooses to do a Table SCAN.

-- In this SPECIFIC case the performance was much better... 
-- but on a larger table performing a table scan everytime would 
-- not be great. The only way to get the best performance is to 
-- create more modular procedures, to use DSE on the Server (in the sp)
-- OR to submit the CORRECT statement (with only the parameters
-- defined by the client (and it's likely that you will NOT want to use
-- sp_EXECutesql to force the statement to be cached - this one you 
-- probably want SQL Server to recompile on each EXECution).
CREATE PROC dbo.GetMemberInfoParamDSE
	@Lastname	varchar(30) = NULL,
	@Firstname	varchar(30) = NULL,
	@member_no	int = NULL
AS
IF @LastName IS NULL AND @FirstName IS NULL AND @Member_no IS NULL
	RAISERROR ('You must supply at least one parameter.', 16, -1)

DECLARE @EXECStr	varchar(1000),
	@MemberNoStr	varchar(100)
SELECT @EXECStr = 'SELECT * FROM dbo.member WHERE ' 

IF @LastName IS NOT NULL
	SELECT @Lastname = 'lastname LIKE ' + QUOTENAME(@lastname, '''')
IF @FirstName IS NOT NULL
	SELECT @Firstname = 'firstname LIKE ' + QUOTENAME(@firstname, '''')
IF @Member_no IS NOT NULL
	SELECT @MemberNoStr = 'member_no = ' + convert(varchar(5), @member_no)

SELECT @EXECStr = @EXECStr + ISNULL(@LastName, ' ') 
	+ 
	CASE
		WHEN @LastName IS NOT NULL AND @FirstName IS NOT NULL
			THEN ' AND '
		ELSE ' '
	END
	+
	ISNULL(@FirstName, ' ') 
	+ 
	CASE
		WHEN (@LastName IS NOT NULL OR @FirstName IS NOT NULL)
					AND @MemberNoStr IS NOT NULL
			THEN ' AND '
		ELSE ' '
	END
	+
	ISNULL(@MemberNoStr, ' ')

EXEC(@EXECStr)
go
EXEC dbo.GetMemberInfoParamDSE 
	@Lastname = 'test', @FirstName = 'Kimberly' 
go
EXEC dbo.GetMemberInfoParamDSE 
	@Firstname = 'Kimberly' 
go
EXEC dbo.GetMemberInfoParamDSE 
	@Firstname = 'Kimberly', @Member_no = 842 
go
EXEC dbo.GetMemberInfoParamDSE 
	@Member_no = 9912 
go
EXEC dbo.GetMemberInfoParamDSE 
	@Lastname = 'Florini', @Member_no = 9912 
go